@@ -12,12 +12,12 @@ use strict;
 require Exporter;
 
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-$VERSION = '0.52';
+$VERSION = '0.53';
 @ISA = qw(Exporter);
 @EXPORT = qw();
 @EXPORT_OK = qw(DBILogin);
 
-use vars qw($Debug $Edit $Echo $DBH $Help $About $CursorWait);
+use vars qw($Debug $Edit $Echo $DBH $Help $About $CursorWait %SQLType);
 $Debug      =0;       # debug level or switch
 $Echo       =1;       # echo printing
 $Edit       =1;       # default edit mode enabled
@@ -25,19 +25,22 @@ $DBH        =undef;   # DBI database Handle
 $Help       =undef;   # 'Help' array ref or sub ref
 $About      =undef;   # 'About' array ref or sub ref
 $CursorWait ='watch'; # Wait cursor type
-
-1;
-
+%SQLType    =         # SQL data type names
+             (0=>'unknown',1=>'CHAR'
+             ,2=>'NUMERIC',3=>'DECIMAL',4=>'INTEGER',5=>'SMALLINT',6=>'FLOAT',7=>'REAL',8=>'DOUBLE'
+             ,9=>'DATETIME',12=>'VARCHAR'
+             ,91=>'DATE',92=>'TIME',93=>'TIMESTAMP');
 
 sub DBILogin {
  my $scr =(ref($_[0]) ? shift : undef);
  my ($dsn, $usr, $psw, $opt, $dbopt) =@_;
     $opt  =$opt   ||'';
     $dbopt=$dbopt || {};
- my $rsp ='';
  my $dbh;
  eval('use DBI');
- my $dlg   =$scr ? $scr : new Tk::MainWindow(-title=>Tk::TM::Lang::txtMsg('Login')); 
+ my $dlg   =$scr ? (ref($scr) eq 'ARRAY' ? $scr->[0] : $scr) 
+                 : new Tk::MainWindow(-title=>Tk::TM::Lang::txtMsg('Login')); 
+ my $rspfd;
  my $dsnlb =$dlg->Label(-text=>Tk::TM::Lang::txtMsg('Database'))
                 ->grid(-row=>0, -column=>0, -sticky=>'w');
  my $dsnfd =$dlg->Entry(-textvariable=>\$dsn)
@@ -53,26 +56,27 @@ sub DBILogin {
                 ->grid(-row=>2, -column=>1, -columnspan=>2, -sticky=>'we');
  my $btnok =$dlg->Button(-text=>Tk::TM::Lang::txtMsg($scr ? 'Login' : 'Ok')
                         ,-command=>
-                           sub{$rsp ='Connecting...';
+                           sub{$rspfd->configure(-text=>'Connecting...');
                                my $curs =$dlg->cget(-cursor);
                                $dlg->configure(-cursor=>$CursorWait);
                                $dlg->update;
                                $dlg->configure(-cursor=>$curs);
                                if (eval {$dbh =DBI->connect($dsn,$usr,$psw,$dbopt)}) 
-                                    {$rsp ='Connected'; 
+                                    {$rspfd->configure(-text=>'Connected'); 
                                      eval {$_[0] =$dsn};
                                      eval {$_[1] =$usr};
                                      eval {$_[2] =$psw};
                                      $DBH =$dbh if $scr || $opt !~/return/i;
                                      $dlg->destroy if !$scr}
-                               else {$rsp =$DBI::errstr}
+                               else {$rspfd->configure(-text=>$DBI::errstr)}
                               }
                         )
                 ->grid(-row=>3, -column=>($scr ? 2 : 1), -sticky=>'we');
  my $btncn =$dlg->Button(-text=>Tk::TM::Lang::txtMsg('Cancel')
                         ,-command=>sub{if(!$scr && $opt =~/return/i) {$dlg->destroy} else {Tk::exit}})
                 ->grid(-row=>3, -column=>2, -sticky=>'we') if !$scr;
- my $rspfd =$dlg->Entry(-textvariable=>\$rsp, -state=>'disabled', -bg=>$dlg->cget(-bg))
+    $rspfd =ref($scr) eq 'ARRAY' ? $scr->[1]
+           :$dlg->Label(-anchor=>'w',-relief=>'sunken')
                 ->grid(-row=>4, -column=>0, -columnspan=>3, -sticky=>'we');
  $dsnfd->bind('<Key-Return>',sub{$btnok->invoke});
  $usrfd->bind('<Key-Return>',sub{$btnok->invoke});
